import java.sql.Array;
import java.sql.SQLOutput;
import java.util.ArrayList;

public class Parser {

    public AstExpression parse (ArrayList<Token> tokenList) {

        AstExpression root;

        root = parseAstExpression(tokenList);

        return root;
    }

    private AstExpression parseAstExpression (ArrayList<Token> tokenList) {

        AstExpression toReturn = new AstExpression();

        // If tokenList contains one item: Expression is a value


        boolean isValue = false;

        if(tokenList.size() == 1 ) {

            toReturn.astValue = parseValue(tokenList);
            //return toReturn;
            isValue = true;

        }

        for (Token t : tokenList) {


            if(t.getTokenString().equals(".")) {
                isValue = true;
            }


            if(t.getTokenString().equals("+") || t.getTokenString().equals("-") || t.getTokenString().equals("*") || t.getTokenString().equals("/")) {
                isValue = false;
                break;
            }

        }


        if(isValue) {
            toReturn.astValue = parseValue(tokenList);
            return toReturn;
        }


        // If tokenList only contains more than one item: Expression must be expression in brackets or binary operation

        // ( 3+4 ) * 5


        toReturn.astBinaryOp = parseBinaryOp(tokenList);

        return toReturn;

    }

    private AstBinaryOp parseBinaryOp (ArrayList<Token> tokenList) {



        AstBinaryOp toReturn = new AstBinaryOp();

        int positionOfFirstTopLevelOperator = 0;
        int numberOfBrackets = 0;
        int i = 0;
        String lastBracket = ")";

        for(Token t : tokenList) {


            if(t.getTokenString().equals("(") || t.getTokenString().equals(")")) {
                numberOfBrackets++;
                lastBracket = t.tokenString;
            }

            if( ((t.getTokenString().equals("+") || t.getTokenString().equals("-") || t.getTokenString().equals("*")
                    || t.getTokenString().equals("/")) && (numberOfBrackets % 2 == 0) ) && lastBracket.equals((")") )) {

                positionOfFirstTopLevelOperator = i;
                break;

            }
            i++;

        }

        // Make two Token lists before and after the expression, parse each as expression

        toReturn.astExpression1 = parseAstExpression(new ArrayList<>(tokenList.subList(0, positionOfFirstTopLevelOperator)));
        toReturn.astOperator = parseOperator(tokenList.get(positionOfFirstTopLevelOperator));
        toReturn.astExpression2 = parseAstExpression(new ArrayList<>(tokenList.subList(positionOfFirstTopLevelOperator + 1, tokenList.size())));


        return toReturn ;
    }

    private AstOperator parseOperator (Token operator) {

        AstOperator astOperator = new AstOperator();
        astOperator.astOperator = operator.tokenString;

        return astOperator;
    }

    private AstValue parseValue (ArrayList<Token> valueTokens) {

        AstValue toReturn = new AstValue();

        boolean containsSpecialToken = false;

        for(Token t : valueTokens) {

            if(t.getTokenType().equals(TokenType.special)) {

                containsSpecialToken = true;

            }

        }

        if(containsSpecialToken) {

            toReturn.astDecimal = parseDecimal(valueTokens);

        } else {

            toReturn.astNumber = parseNumber(valueTokens);

        }
        return toReturn;
    }

    private AstDecimal parseDecimal (ArrayList<Token> decimalTokens) {

        AstDecimal toReturn = new AstDecimal();

        int i = 0;
        int positionOfDecimalPoint = -1;

        for(Token t : decimalTokens) {

            if(t.getTokenType().equals(TokenType.special)) {
                positionOfDecimalPoint = i;
                break;
            }

            i++;

        }

        // parse digits before comma as list and save it into toReturn.before



        if (positionOfDecimalPoint == 1) {

            ArrayList<Token> temp = new ArrayList<>();
            temp.add(decimalTokens.get(0));
            toReturn.astDigitBeforeComma = parseNumber(temp);

        } else {

            toReturn.astDigitBeforeComma = parseNumber(new ArrayList<> (decimalTokens.subList(0, positionOfDecimalPoint - 1)));
        }


        if (positionOfDecimalPoint +  1 - decimalTokens.size() == 0) {

            ArrayList<Token> temp = new ArrayList<>();
            temp.add(decimalTokens.get(decimalTokens.size() - 1));
            toReturn.astDigitAfterComma = parseNumber(temp);

        } else {

            toReturn.astDigitAfterComma = parseNumber(new ArrayList<> (decimalTokens.subList(positionOfDecimalPoint +  1, decimalTokens.size())));

        }
        



        return toReturn;
    }

    private AstNumber parseNumber (ArrayList<Token> numberTokens) {

        AstNumber toReturn = new AstNumber();

        // take first Token from list and parse as digit without zero

        toReturn.astDigitWoz = parseDigitWoZ(numberTokens.get(0));

        if(numberTokens.size() == 1) {
            return toReturn;
        }

        numberTokens.stream()
                .skip(1)
                .forEach(token -> {
                    AstDigit temp = parseAstDigit(token);
                    toReturn.astDigits.add(temp);
                });

        return toReturn;

    }

    private AstDigitWoZ parseDigitWoZ (Token digitWoZ) {

        AstDigitWoZ toReturn = new AstDigitWoZ();

        int tokenContentAsString;

        try {
            tokenContentAsString = Integer.parseInt(digitWoZ.tokenString);
        } catch (NumberFormatException error) {
            System.out.println("Can't parse string in AstDigitWoZ");
            throw error;
        }


        toReturn.astDigitWozContent = tokenContentAsString;

        return toReturn;
    }

    private AstDigit parseAstDigit (Token digit) {

        AstDigit toReturn = new AstDigit();

        int tokenContentAsString;


        try {
            tokenContentAsString = Integer.parseInt(digit.tokenString);
        } catch (NumberFormatException error) {
            System.out.println("Can't parse string in AstDigit");
            throw error;
        }



        if(tokenContentAsString == 0) {
            toReturn.astDigit = 0;
            return toReturn;
        }

        toReturn.astDigitWoZ = parseDigitWoZ(digit);

        return toReturn;
    }


}
